<!--
This is the directive for assigning tokens to users.
-->
<div class="form-group">
    <label for="serial" translate>Serial</label>
    <input name="serial" type="text" ng-model="newTokenObject.serial"
           placeholder="{{ 'start typing a serial number of a token that is notassigned, yet.'|translate}} "
           typeahead="serial for serial in loadSerials($viewValue)"
           typeahead-loading="loadingSerials" class="form-control" >
    <i ng-show="loadingSerials" class="glyphicon glyphicon-refresh"
            translate>Loading serials...</i>
</div>
<div class="form-group">
    <label for="otppin" translate>PIN</label>
    <input name="otppin" ng-model="newTokenObject.pin"
            type=password class="form-control"
            euqals="{{pin2}}"
            placeholder="{{ 'Type a password'|translate }}">
    <input name="otppin2" ng-model="pin2"
            type=password class="form-control"
            equals="{{newTokenObject.pin}}"
            placeholder="{{ 'Repeat password'|translate }}">
</div>
