<p translate>
    The certificate was enrolled successfully.
</p>
<p>
    <a download="{{ enrolledToken.serial }}.pem"
       ng-href="{{ certificateBlob }}">
        <span class="glyphicon glyphicon-bookmark"
                          aria-hidden="true"></span>
        <translate>Download the certificate</translate></a>.
</p>
<pre>
{{ enrolledToken.certificate }}
</pre>
