<div class="well">

    <div class="container">
        <form name="formResponse" class="form-signin" role="form" novalidate>
            <div class="center">
                <p>{{ challenge_message }}</p>
            </div>
            <div class="center"
                ng-show="image">
                <img width="250" src="{{ image }}"/>
            </div>

            <div ng-hide="hideResponseInput">
                <h2 class="form-signin-heading" translate>Response</h2>

                <input type="hidden" name="username" id="username"
                       ng-model="login.username">
                <input type="hidden" name="transactionid" id="transactionid"
                       ng-model="transactionid">


                <label for="password" class="sr-only" translate>Response</label>
                <input name="password" type="password"
                       id="password" class="form-control"
                       ng-model="login.password" required
                       placeholder="{{ challenge_message }}"/>

                <div class="text-center">
                    <button ng-click="authenticate()"
                            ng-disabled="authForm.$invalid"
                            class="btn btn-primary btn-block" translate>Log In
                    </button>
                </div>
            </div>
        </form>
    </div>
</div>
