{% include 'header.html' %}

<!-- Include menu -->

<ng-include src="'{{ instance }}/static/templates/menu.html'">
</ng-include>
<div class="container">
    <div ui-view class="slide"></div>
</div>

<input type=hidden id=REMOTE_USER value="{{ remote_user }}">

{% raw %}
<div class="navbar navbar-inverse navbar-fixed-bottom">
    <div class="container">
        <a ng-click="about()"
            class="navbar-text pull-left">
            privacyIDEA
            {{ privacyideaVersionNumber }}
        </a>	
        <a class="navbar-btn btn btn-success pull-right"
           href="https://netknights.it/leistungen/support/"
           target="_external">
            <translate>Support</translate>
        </a>
        <a class="navbar-text pull-right"
                href="https://groups.google.com/forum/#!forum/privacyidea"
                target="_external">
            privacyIDEA google group</a>
        <a class="navbar-text pull-right"
           ng-show="privacyideaVersionNumber"
                href="http://privacyidea.readthedocs.org/en/v{{ privacyideaVersionNumber }}"
                target="documentation">
            <translate>Online Documentation</translate>
        </a>
    </div>
</div>
{% endraw %}

{% include 'footer.html' %}
