<h2 translate>RADIUS Token settings</h2>
<p class="help-block" translate>
    The RADIUS token forwards the authentication request to another RADIUS
    server.
</p>

<div class="form-group">
    <label for="radiusServer" translate>Default RADIUS Server</label>
    <input type="text"
           required
           class="form-control"
           placeholder="your.radius.server:1812"
           ng-model="form['radius.server']" name="radiusServer">
</div>

<div class="form-group">
    <label for="radiusDict" translate>RADIUS dictionary file</label>
    <input type="text"
           required
           class="form-control"
           placeholder="/etc/privacyidea/dictionary"
           ng-model="form['radius.dictfile']" name="radiusDict">
</div>

<div class="form-group">
    <label for="radiusSecret" translate>Default RADIUS Secret</label>
    <input type="password"
           required
           equals="{{ secret2 }}"
           class="form-control"
           ng-model="form['radius.secret']" name="radiusSecret">
    <input type="password"
           equals="{{ form['radius.secret'] }}"
           class="form-control"
           ng-model="secret2" name="radiusSecret2">
</div>
